-- ==============================================================
-- FILE : FPDP.VHO
-- ==============================================================
--
-- Instantiation templates for the FPDP example for HERON-FPGA and
-- HERON-IO modules.
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    16-12-02    First Written
--
-- ==============================================================


-- The following code must appear in the VHDL architecture header

-- Begin the CUT here for the FPDP COMPONENT declaration
  component FPDP
    port (
      -- FPDP Control Inputs and Output
      PSTROBE_PIN  : in  std_logic;
      DVALID_PIN   : in  std_logic;
      SYNC_PIN     : in  std_logic;
      SUSPEND_PIN  : in  std_logic;
      DIR_PIN      : in  std_logic;
      PIO1_PIN     : in  std_logic;
      PIO2_PIN     : in  std_logic;
      NRDY_PIN     : out std_logic;
      NRDY_PIN_EN  : out std_logic;
      -- FPDP Data Inputs
      DIN_PINS     : in  std_logic_vector(31 downto 0);
      -- FPDP Control Signals
      FPDP_CLOCK   : out std_logic;
      FPDP_DVALID  : out std_logic;
      FPDP_SYNC    : out std_logic;
      FPDP_SUSPEND : out std_logic;
      FPDP_DIR     : out std_logic;
      FPDP_PIO1    : out std_logic;
      FPDP_PIO2    : out std_logic;
      FPDP_READY   : in  std_logic;
      -- FPDP Data Signals
      FPDP_DIN     : out std_logic_vector(31 downto 0)
    );
  end component;
-- End CUT here


-- The following code must appear in the VHDL architecture body
-- Please note, you will need to substitute your own instance name
-- and net names according to your own design requirements

-- Begin the CUT here for FPDP component instantiation
  iFPDP : FPDP
    port map (
      -- FPDP Control Inputs and Output
      PSTROBE_PIN  => QTTL,
      DVALID_PIN   => CONN_C_IN(2),
      SYNC_PIN     => CONN_C_IN(3),
      SUSPEND_PIN  => CONN_C_IN(6),
      DIR_PIN      => CONN_C_IN(7),
      PIO1_PIN     => CONN_C_IN(4),
      PIO2_PIN     => CONN_C_IN(5),
      NRDY_PIN     => CONN_C_OUT(8),
      NRDY_PIN_EN  => CONN_C_EN(8),
      -- FPDP Data Inputs
      DIN_PINS     => FPDP_DIN_PINS,
      -- FPDP Control Signals
      FPDP_CLOCK   => FPDP_CLOCK,
      FPDP_DVALID  => FPDP_DVALID,
      FPDP_SYNC    => FPDP_SYNC,
      FPDP_SUSPEND => FPDP_SUSPEND,
      FPDP_DIR     => FPDP_DIR,
      FPDP_PIO1    => FPDP_PIO1,
      FPDP_PIO2    => FPDP_PIO2,
      FPDP_READY   => FPDP_READY,
      -- FPDP Data Signals
      FPDP_DIN     => FPDP_DIN );
-- End CUT here


-- ==============================================================
--
-- The following code is for users of the HERON-FPGA4 module only
--
-- This code should be used in the top.vhd file (part of the
-- /Common directory for the HERON-FPGA4), in order to implement
-- a LVPECL_33 input buffer for the FPDP PSTROBE signal.
--
-- ==============================================================


-- The following code must appear in the VHDL architecture header
-- for the file 'top.vhd'

-- Begin the CUT here for the LVPECL Input Buffer COMPONENT declaration
  component IBUFDS_LVPECL_33
    port(
      I  : in  std_logic;
      IB : in  std_logic;
      O  : out std_logic );
  end component;
-- End CUT here


-- The following code must appear in the VHDL architecture body for
-- the file 'top.vhd'

-- Begin the CUT here for LVPECL input buffer component instantiation
  lvpecl_ibufg : IBUFDS_LVPECL_33 port map (I=>dio_c(10), IB=>dio_c(9), O=>PSTROBE_PIN);
-- End CUT here

-- As the above instantion uses the dio_c(9) and dio_c(10) signals in this example,
-- the previous two IOBUF buffers on these signals must be deleted
-- (instances iDIOC9 and iDIOC10)

-- PLEASE NOTE : you will need to add a new signal in the port map of the USER_AP component
-- so that the new PSTROBE_PIN can be connected to the user application level
