-- ==============================================================
-- FILE : FPDP.VHD  -  can be modified by the USER
-- ==============================================================
--
-- This file is part of the FPDP example for HERON-FPGA and
-- HERON-IO modules.
--
-- This file can be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : FPDP
--  Date        : 16/12/2002
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : FPDP Data and Control Inputs
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    16-12-02    First Written
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;

--pragma translate_off
library UNISIM;
  use UNISIM.vcomponents.all;
--pragma translate_on


entity FPDP is
  port (
    -- FPDP Control Inputs and Output
    PSTROBE_PIN  : in  std_logic;
    DVALID_PIN   : in  std_logic;
    SYNC_PIN     : in  std_logic;
    SUSPEND_PIN  : in  std_logic;
    DIR_PIN      : in  std_logic;
    PIO1_PIN     : in  std_logic;
    PIO2_PIN     : in  std_logic;
    NRDY_PIN     : out std_logic;
    NRDY_PIN_EN  : out std_logic;
    -- FPDP Data Inputs
    DIN_PINS     : in  std_logic_vector(31 downto 0);
    -- FPDP Control Signals
    FPDP_CLOCK   : out std_logic;
    FPDP_DVALID  : out std_logic;
    FPDP_SYNC    : out std_logic;
    FPDP_SUSPEND : out std_logic;
    FPDP_DIR     : out std_logic;
    FPDP_PIO1    : out std_logic;
    FPDP_PIO2    : out std_logic;
    FPDP_READY   : in  std_logic;
    -- FPDP Data Signals
    FPDP_DIN     : out std_logic_vector(31 downto 0)
  );
end FPDP;


architecture RTL of FPDP is

  component BUFG
    port(I : in  std_logic;
         O : out std_logic );
  end component;

  component FDC
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic;
      CLR : in  std_logic );
  end component;

  attribute IOB : string;

  -- Input Flip-Flops for FPDP Control Signals
  attribute IOB of iCTRL0 : label is "TRUE";
  attribute IOB of iCTRL1 : label is "TRUE";
  attribute IOB of iCTRL2 : label is "TRUE";
  attribute IOB of iCTRL3 : label is "TRUE";
  attribute IOB of iCTRL4 : label is "TRUE";
  attribute IOB of iCTRL5 : label is "TRUE";

  -- Input Flip-Flops for FPDP Data Signals
  attribute IOB of iDIN0  : label is "TRUE";
  attribute IOB of iDIN1  : label is "TRUE";
  attribute IOB of iDIN2  : label is "TRUE";
  attribute IOB of iDIN3  : label is "TRUE";
  attribute IOB of iDIN4  : label is "TRUE";
  attribute IOB of iDIN5  : label is "TRUE";
  attribute IOB of iDIN6  : label is "TRUE";
  attribute IOB of iDIN7  : label is "TRUE";
  attribute IOB of iDIN8  : label is "TRUE";
  attribute IOB of iDIN9  : label is "TRUE";
  attribute IOB of iDIN10 : label is "TRUE";
  attribute IOB of iDIN11 : label is "TRUE";
  attribute IOB of iDIN12 : label is "TRUE";
  attribute IOB of iDIN13 : label is "TRUE";
  attribute IOB of iDIN14 : label is "TRUE";
  attribute IOB of iDIN15 : label is "TRUE";
  attribute IOB of iDIN16 : label is "TRUE";
  attribute IOB of iDIN17 : label is "TRUE";
  attribute IOB of iDIN18 : label is "TRUE";
  attribute IOB of iDIN19 : label is "TRUE";
  attribute IOB of iDIN20 : label is "TRUE";
  attribute IOB of iDIN21 : label is "TRUE";
  attribute IOB of iDIN22 : label is "TRUE";
  attribute IOB of iDIN23 : label is "TRUE";
  attribute IOB of iDIN24 : label is "TRUE";
  attribute IOB of iDIN25 : label is "TRUE";
  attribute IOB of iDIN26 : label is "TRUE";
  attribute IOB of iDIN27 : label is "TRUE";
  attribute IOB of iDIN28 : label is "TRUE";
  attribute IOB of iDIN29 : label is "TRUE";
  attribute IOB of iDIN30 : label is "TRUE";
  attribute IOB of iDIN31 : label is "TRUE";

  -- FPDP Clock Signal
  signal CLOCK   : std_logic;
  -- FPDP Control Signals
  signal DVALID  : std_logic;
  signal SYNC    : std_logic;
  signal SUSPEND : std_logic;
  signal DIR     : std_logic;
  signal PIO1    : std_logic;
  signal PIO2    : std_logic;
  signal NRDY    : std_logic;
  -- FPDP Data Signals
  signal DIN     : std_logic_vector(31 downto 0);

begin

  iBUFG : BUFG port map(I=> PSTROBE_PIN, O => CLOCK);

  FPDP_CLOCK   <= CLOCK;
  FPDP_DVALID  <= NOT DVALID;
  FPDP_SYNC    <= SYNC;
  FPDP_SUSPEND <= SUSPEND;
  FPDP_DIR     <= DIR;
  FPDP_PIO1    <= PIO1;
  FPDP_PIO2    <= PIO2;
  FPDP_DIN     <= DIN;

  NRDY_PIN    <= '0';
  NRDY_PIN_EN <= FPDP_READY;

  -- Instanciate these flip-flops to ensure they will always be IFDs
  iCTRL0 : FDC port map (DVALID,  D=>DVALID_PIN,   C=>CLOCK, CLR=>'0');  -- IOB
  iCTRL1 : FDC port map (SYNC,    D=>SYNC_PIN,     C=>CLOCK, CLR=>'0');  -- IOB
  iCTRL2 : FDC port map (SUSPEND, D=>SUSPEND_PIN,  C=>CLOCK, CLR=>'0');  -- IOB
  iCTRL3 : FDC port map (DIR,     D=>DIR_PIN,      C=>CLOCK, CLR=>'0');  -- IOB
  iCTRL4 : FDC port map (PIO1,    D=>PIO1_PIN,     C=>CLOCK, CLR=>'0');  -- IOB
  iCTRL5 : FDC port map (PIO2,    D=>PIO2_PIN,     C=>CLOCK, CLR=>'0');  -- IOB

  -- Instanciate these flip-flops to ensure they will always be IFDs
  iDIN0  : FDC port map (DIN(0),  D=>DIN_PINS(0),  C=>CLOCK, CLR=>'0');  -- IOB
  iDIN1  : FDC port map (DIN(1),  D=>DIN_PINS(1),  C=>CLOCK, CLR=>'0');  -- IOB
  iDIN2  : FDC port map (DIN(2),  D=>DIN_PINS(2),  C=>CLOCK, CLR=>'0');  -- IOB
  iDIN3  : FDC port map (DIN(3),  D=>DIN_PINS(3),  C=>CLOCK, CLR=>'0');  -- IOB
  iDIN4  : FDC port map (DIN(4),  D=>DIN_PINS(4),  C=>CLOCK, CLR=>'0');  -- IOB
  iDIN5  : FDC port map (DIN(5),  D=>DIN_PINS(5),  C=>CLOCK, CLR=>'0');  -- IOB
  iDIN6  : FDC port map (DIN(6),  D=>DIN_PINS(6),  C=>CLOCK, CLR=>'0');  -- IOB
  iDIN7  : FDC port map (DIN(7),  D=>DIN_PINS(7),  C=>CLOCK, CLR=>'0');  -- IOB

  iDIN8  : FDC port map (DIN(8),  D=>DIN_PINS(8),  C=>CLOCK, CLR=>'0');  -- IOB
  iDIN9  : FDC port map (DIN(9),  D=>DIN_PINS(9),  C=>CLOCK, CLR=>'0');  -- IOB
  iDIN10 : FDC port map (DIN(10), D=>DIN_PINS(10), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN11 : FDC port map (DIN(11), D=>DIN_PINS(11), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN12 : FDC port map (DIN(12), D=>DIN_PINS(12), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN13 : FDC port map (DIN(13), D=>DIN_PINS(13), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN14 : FDC port map (DIN(14), D=>DIN_PINS(14), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN15 : FDC port map (DIN(15), D=>DIN_PINS(15), C=>CLOCK, CLR=>'0');  -- IOB

  iDIN16 : FDC port map (DIN(16), D=>DIN_PINS(16), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN17 : FDC port map (DIN(17), D=>DIN_PINS(17), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN18 : FDC port map (DIN(18), D=>DIN_PINS(18), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN19 : FDC port map (DIN(19), D=>DIN_PINS(19), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN20 : FDC port map (DIN(20), D=>DIN_PINS(20), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN21 : FDC port map (DIN(21), D=>DIN_PINS(21), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN22 : FDC port map (DIN(22), D=>DIN_PINS(22), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN23 : FDC port map (DIN(23), D=>DIN_PINS(23), C=>CLOCK, CLR=>'0');  -- IOB

  iDIN24 : FDC port map (DIN(24), D=>DIN_PINS(24), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN25 : FDC port map (DIN(25), D=>DIN_PINS(25), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN26 : FDC port map (DIN(26), D=>DIN_PINS(26), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN27 : FDC port map (DIN(27), D=>DIN_PINS(27), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN28 : FDC port map (DIN(28), D=>DIN_PINS(28), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN29 : FDC port map (DIN(29), D=>DIN_PINS(29), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN30 : FDC port map (DIN(30), D=>DIN_PINS(30), C=>CLOCK, CLR=>'0');  -- IOB
  iDIN31 : FDC port map (DIN(31), D=>DIN_PINS(31), C=>CLOCK, CLR=>'0');  -- IOB

end RTL;
