-- ==============================================================
-- FILE : GD14.VHD
-- ==============================================================
--
-- This file provides example VHDL code for using your FPGA
-- module with a HEGD14.
--
-- Copy and Paste the lines of VHDL from this file into your
-- own project.
--
-- ==============================================================
--
--  Date        : 15/09/2004
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : GD14 Example Code
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    15-09-04    First Written
--
-- ==============================================================


  -- The following code must appear in the VHDL architecture header:

  -- Unsigned, positive data to be sent to HEGD14
  signal GD14_UNS_DATA : std_logic_vector(13 downto 0);

  -- Data Enable to qualify data presented on 'GD14_DATA'
  signal GD14_CH_EN : std_logic;

  -- Last Channel Indicator
  signal GD14_LAST_CH : std_logic;

  signal DAC_DATA   : std_logic_vector(13 downto 0);
  signal DAC_TOGGLE : std_logic;


  -- The following code must appear in the VHDL architecture body.

  -- PLEASE NOTE. All signals defined below must be used in the HERON
  -- Output FIFO Clock Domain. That is, all are clocked using the signal
  -- FCLK_G.

  -- For this example VHDL, HERON Output FIFO 0 is presented with the
  -- data stream for the HEGD14.

  -- The signals OUTFIFO_WRITE and OUTFIFO_D are used. These signals
  -- are found in the port list of the USER-AP entity in your project.

  -- The signal GD14_CH_EN must be set high in every clock cycle where
  -- a data sample is driven on the bus GD14_UNS_DATA. You must generate
  -- the necessary logic to control the operation of GD14_CH_EN.
  GD14_CH_EN <=   ;

  -- The HEGD14 can output data on up to 8 channels. When updating
  -- multiple channels, a block of samples will be sent together,
  -- incrementing from channel 0. For the last channel to be updated,
  -- the signal GD14_LAST_CH must be set high. You must generate the
  -- necessary logic to control the operation of GD14_LAST_CH.
  GD14_LAST_CH <=   ;

  -- For single channel operation on Channel 0, tie GD14_LAST_CH high.

  -- Invert Data Presented to HEGD14. This is because the HEGD14
  -- expects negative unsigned data.
  DAC_DATA <= NOT GD14_UNS_DATA;

  -- Logic to handle packing samples into 32-bit words
  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      OUTFIFO_D <= (others=>'0');
    elsif rising_edge(FCLK_G) then
      if GD14_CH_EN='1' and DAC_TOGGLE='0' then
        OUTFIFO_D(15 downto 0)  <= GD14_LAST_CH & '0' & DAC_DATA;
      end if;
      if GD14_CH_EN='1' and DAC_TOGGLE='1' then
        OUTFIFO_D(31 downto 16) <= GD14_LAST_CH & '0' & DAC_DATA;
      end if;
    end if;
  end process;

  -- Control signal to monitor which half word is next to be updated
  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      DAC_TOGGLE <= '0';
    elsif rising_edge(FCLK_G) then
      if GD14_CH_EN='1' then
        DAC_TOGGLE <= '0';
        if DAC_TOGGLE='0' then
          DAC_TOGGLE <= '1';
        end if;
      end if;
    end if;
  end process;

  -- HERON Output FIFO Write Enable logic
  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      OUTFIFO_WRITE(0) <= '0';
    elsif rising_edge(FCLK_G) then
      OUTFIFO_WRITE(0) <= '0';
      if GD14_CH_EN='1' and DAC_TOGGLE='1' then
        OUTFIFO_WRITE(0) <= '1';
      end if;
    end if;
  end process;

  OUTFIFO_WRITE(5 downto 1) <= (others=>'0');
